package net.wanji.opt.kafka;

import com.fasterxml.jackson.core.type.TypeReference;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class ConsumerHandler implements KafkaListenerErrorHandler {

    private final CrossDirDataRealtimeMapper crossDirDataRealTimeMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final CrossDataRealtimeMapper crossDataRealTimeMapper;
    private final CrossDataHistMapper crossDataHistMapper;

    @Transactional
    @KafkaListener(topics = {"cross_real_time_lane_data"})
    public void receiveCrossRealTimeLaneData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String crossRealTimeLaneData = String.valueOf(record.value());
            if (StringUtils.isNotBlank(crossRealTimeLaneData)) {
                Map<String, List<CrossLaneDataRealTimePO>> map = JacksonUtils.getInstance().readValue(crossRealTimeLaneData, new TypeReference<Map<String, List<CrossLaneDataRealTimePO>>>() {});
                if (Objects.nonNull(map) && !map.isEmpty()) {
                    List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = map.get("crossRealTimeLaneData");
                    if (!CollectionUtils.isEmpty(crossLaneDataRealTimePOS)) {
                        crossLaneDataRealTimeMapper.deleteBatch(crossLaneDataRealTimePOS.stream().map(CrossLaneDataRealTimePO::getCrossId).collect(Collectors.toList()));
                        crossLaneDataRealTimeMapper.insertBatch(crossLaneDataRealTimePOS);
                        List<CrossLaneDataHistPO> crossLaneDataHistPOS = new ArrayList<>(crossLaneDataRealTimePOS.size());
                        BeanListUtils.populateList(crossLaneDataRealTimePOS, crossLaneDataHistPOS, CrossLaneDataHistPO.class);
                        crossLaneDataHistMapper.insertBatch(crossLaneDataHistPOS);
                    }
                }
            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("kafka消费数据插入路口车道实时表异常", e);
            throw new Exception(e);
        }
    }

    @Transactional
    @KafkaListener(topics = {"cross_real_time_data"})
    public void receiveCrossRealTimeData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String crossRealTimeData = String.valueOf(record.value());
            if (StringUtils.isNotBlank(crossRealTimeData)) {
                Map<String, List<CrossDataRealtimePO>> map = JacksonUtils.getInstance().readValue(crossRealTimeData, new TypeReference<Map<String, List<CrossDataRealtimePO>>>() {});
                if (Objects.nonNull(map) && !map.isEmpty()) {
                    List<CrossDataRealtimePO> crossDataRealtimePOS = map.get("crossRealTimeData");
                    if (!CollectionUtils.isEmpty(crossDataRealtimePOS)) {
                        crossDataRealTimeMapper.deleteBatch(crossDataRealtimePOS.stream().map(CrossDataRealtimePO::getCrossId).collect(Collectors.toList()));
                        crossDataRealTimeMapper.insertBatch(crossDataRealtimePOS);
                        List<CrossDataHistPO> crossDataHistPOS = new ArrayList<>(crossDataRealtimePOS.size());
                        BeanListUtils.populateList(crossDataRealtimePOS, crossDataHistPOS, CrossDataHistPO.class);
                        crossDataHistMapper.insertBatch(crossDataHistPOS);
                    }
                }

            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("kafka消费数据插入路口实时表异常", e);
            throw new Exception(e);
        }
    }

    @Transactional
    @KafkaListener(topics = {"cross_real_time_dir_data"})
    public void receiveCrossRealTimeDirData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String crossRealTimeDirData = String.valueOf(record.value());
            if (StringUtils.isNotBlank(crossRealTimeDirData)) {
                Map<String, List<CrossDirDataRealtimePO>> map = JacksonUtils.getInstance().readValue(crossRealTimeDirData, new TypeReference<Map<String, List<CrossDirDataRealtimePO>>>() {});
                if (Objects.nonNull(map) && !map.isEmpty()) {
                    List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = map.get("crossRealTimeDirData");
                    if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
                        crossDirDataRealTimeMapper.deleteBatch(crossDirDataRealtimePOS.stream().map(CrossDirDataRealtimePO::getCrossId).collect(Collectors.toList()));
                        crossDirDataRealTimeMapper.insertBatch(crossDirDataRealtimePOS);
                        List<CrossDirDataHistPO> list = new ArrayList<>(crossDirDataRealtimePOS.size());
                        BeanListUtils.populateList(crossDirDataRealtimePOS, list, CrossDirDataHistPO.class);
                        crossDirDataHistMapper.insertBatch(list);
                    }
                }
            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("kafka消费数据插入路口方向实时表异常", e);
            throw new Exception(e);
        }
    }

    @Transactional
    @KafkaListener(topics = {"cross_real_time_turn_data"})
    public void receiveCrossRealTimeTurnData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String crossRealTimeTurnData = String.valueOf(record.value());
            if (StringUtils.isNotBlank(crossRealTimeTurnData)) {
                Map<String, List<CrossTurnDataRealtimePO>> map = JacksonUtils.getInstance().readValue(crossRealTimeTurnData, new TypeReference<Map<String, List<CrossTurnDataRealtimePO>>>() {});
                if (Objects.nonNull(map) && !map.isEmpty()) {
                    List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = map.get("crossRealTimeTurnData");
                    if (!CollectionUtils.isEmpty(crossTurnDataRealtimePOS)) {
                        List<String> crossIds = crossTurnDataRealtimePOS.stream().map(CrossTurnDataRealtimePO::getCrossId).distinct().collect(Collectors.toList());
                        crossTurnDataRealTimeMapper.deleteBatch(crossIds);
                        crossTurnDataRealTimeMapper.insertBatch(crossTurnDataRealtimePOS);
                        List<CrossTurnDataHistPO> list = new ArrayList<>(crossTurnDataRealtimePOS.size());
                        BeanListUtils.populateList(crossTurnDataRealtimePOS, list, CrossTurnDataHistPO.class);
                        crossTurnDataHistMapper.insertBatch(list);
                    }
                }
            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("kafka消费数据插入路口转向实时表异常", e);
            throw new Exception(e);
        }
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
